@@ -19,7 +19,7 @@ exposure
     Image intensity adjustment, e.g., histogram equalization, etc.
 feature
     Feature detection and extraction, e.g., texture analysis corners, etc.
-filter
+filters
     Sharpening, edge finding, rank filters, thresholding, etc.
 graph
     Graph-theoretic operations, e.g., shortest paths.
@@ -56,36 +56,26 @@ img_as_ubyte
 
 """
 
-import os.path as _osp
-import imp as _imp
-import functools as _functools
-import warnings as _warnings
-from skimage._shared.utils import deprecated as _deprecated
+import os.path as osp
+import imp
+import functools
+import warnings
+import sys
 
-pkg_dir = _osp.abspath(_osp.dirname(__file__))
-data_dir = _osp.join(pkg_dir, 'data')
+pkg_dir = osp.abspath(osp.dirname(__file__))
+data_dir = osp.join(pkg_dir, 'data')
+
+__version__ = '0.12.3'
 
 try:
-    from .version import version as __version__
+    imp.find_module('nose')
 except ImportError:
-    __version__ = "unbuilt-dev"
-del version
-
-
-try:
-    _imp.find_module('nose')
-except ImportError:
-    def _test(verbose=False):
+    def _test(doctest=False, verbose=False):
         """This would run all unit tests, but nose couldn't be
         imported so the test suite can not run.
         """
         raise ImportError("Could not load nose. Unit tests not available.")
 
-    def _doctest(verbose=False):
-        """This would run all doc tests, but nose couldn't be
-        imported so the test suite can not run.
-        """
-        raise ImportError("Could not load nose. Doctests not available.")
 else:
     def _test(doctest=False, verbose=False):
         """Run all unit tests."""
@@ -97,8 +87,8 @@ else:
             args.extend(['--with-doctest', '--ignore-files=^\.',
                          '--ignore-files=^setup\.py$$', '--ignore-files=test'])
             # Make sure warnings do not break the doc tests
-            with _warnings.catch_warnings():
-                _warnings.simplefilter("ignore")
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
                 success = nose.run('skimage', argv=args)
         else:
             success = nose.run('skimage', argv=args)
@@ -112,60 +102,65 @@ else:
 # do not use `test` as function name as this leads to a recursion problem with
 # the nose test suite
 test = _test
-test_verbose = _functools.partial(test, verbose=True)
+test_verbose = functools.partial(test, verbose=True)
 test_verbose.__doc__ = test.__doc__
-doctest = _functools.partial(test, doctest=True)
+doctest = functools.partial(test, doctest=True)
 doctest.__doc__ = doctest.__doc__
-doctest_verbose = _functools.partial(test, doctest=True, verbose=True)
+doctest_verbose = functools.partial(test, doctest=True, verbose=True)
 doctest_verbose.__doc__ = doctest.__doc__
 
 
-class _Log(Warning):
-    pass
+# Logic for checking for improper install and importing while in the source
+# tree when package has not been installed inplace.
+# Code adapted from scikit-learn's __check_build module.
+_INPLACE_MSG = """
+It appears that you are importing a local scikit-image source tree. For
+this, you need to have an inplace install. Maybe you are in the source
+directory and you need to try from another location."""
+
+_STANDARD_MSG = """
+Your install of scikit-image appears to be broken.
+Try re-installing the package following the instructions at:
+http://scikit-image.org/docs/stable/install.html """
 
 
-class _FakeLog(object):
-    def __init__(self, name):
-        """
-        Parameters
-        ----------
-        name : str
-            Name of the log.
-        repeat : bool
-            Whether to print repeating messages more than once (False by
-            default).
-        """
-        self._name = name
+def _raise_build_error(e):
+    # Raise a comprehensible error
+    local_dir = osp.split(__file__)[0]
+    msg = _STANDARD_MSG
+    if local_dir == "skimage":
+        # Picking up the local install: this will work only if the
+        # install is an 'inplace build'
+        msg = _INPLACE_MSG
+    raise ImportError("""%s
+It seems that scikit-image has not been built correctly.
+%s""" % (e, msg))
 
-        warnings.simplefilter("always", _Log)
+try:
+    # This variable is injected in the __builtins__ by the build
+    # process. It used to enable importing subpackages of skimage when
+    # the binaries are not built
+    __SKIMAGE_SETUP__
+except NameError:
+    __SKIMAGE_SETUP__ = False
 
-        self._warnings = _warnings
-
-    def _warn(self, msg, wtype):
-        self._warnings.warn('%s: %s' % (wtype, msg), _Log)
-
-    def debug(self, msg):
-        self._warn(msg, 'DEBUG')
-
-    def info(self, msg):
-        self._warn(msg, 'INFO')
-
-    def warning(self, msg):
-        self._warn(msg, 'WARNING')
-
-    warn = warning
-
-    def error(self, msg):
-        self._warn(msg, 'ERROR')
-
-    def critical(self, msg):
-        self._warn(msg, 'CRITICAL')
-
-    def addHandler(*args):
-        pass
-
-    def setLevel(*args):
-        pass
+if __SKIMAGE_SETUP__:
+    sys.stderr.write('Partial import of skimage during the build process.\n')
+    # We are not importing the rest of the scikit during the build
+    # process, as it may not be compiled yet
+else:
+    try:
+        from ._shared import geometry
+        del geometry
+    except ImportError as e:
+        _raise_build_error(e)
+    from .util.dtype import *
 
 
-from .util.dtype import *
+if sys.version.startswith('2.6'):
+    msg = ("Python 2.6 is deprecated and will not be supported in "
+           "scikit-image 0.13+")
+    warnings.warn(msg, stacklevel=2)
+
+
+del warnings, functools, osp, imp, sys
