@@ -13,30 +13,38 @@ Gems - authors, editors, publishers, or webmasters - are to be held
 responsible.  Basically, don't be a jerk, and remember that anything free
 comes with no guarantee.
 """
+from __future__ import division
+import numbers
 import numpy as np
-import skimage
-from skimage import color
-from skimage.exposure import rescale_intensity
-from skimage.util import view_as_blocks
+from .. import img_as_float, img_as_uint
+from ..color.adapt_rgb import adapt_rgb, hsv_value
+from ..exposure import rescale_intensity
+from .._shared.utils import skimage_deprecation, warn
+
+NR_OF_GREY = 2 ** 14  # number of grayscale levels to use in CLAHE algorithm
 
 
-MAX_REG_X = 16  # max. # contextual regions in x-direction */
-MAX_REG_Y = 16  # max. # contextual regions in y-direction */
-NR_OF_GREY = 16384  # number of grayscale levels to use in CLAHE algorithm
+@adapt_rgb(hsv_value)
+def equalize_adapthist(image, ntiles_x=None, ntiles_y=None, clip_limit=0.01,
+                       nbins=256, kernel_size=None):
+    """Contrast Limited Adaptive Histogram Equalization (CLAHE).
 
-
-def equalize_adapthist(image, ntiles_x=8, ntiles_y=8, clip_limit=0.01,
-                       nbins=256):
-    """Contrast Limited Adaptive Histogram Equalization.
+    An algorithm for local contrast enhancement, that uses histograms computed
+    over different tile regions of the image. Local details can therefore be
+    enhanced even in regions that are darker or lighter than most of the image.
 
     Parameters
     ----------
     image : array-like
         Input image.
-    ntiles_x : int, optional
-        Number of tile regions in the X direction.  Ranges between 2 and 16.
-    ntiles_y : int, optional
-        Number of tile regions in the Y direction.  Ranges between 2 and 16.
+    kernel_size: integer or 2-tuple
+        Defines the shape of contextual regions used in the algorithm.
+        If an integer is given, the shape will be a square of
+        sidelength given by this value.
+    ntiles_x : int, optional (deprecated in favor of ``kernel_size``)
+        Number of tile regions in the X direction (horizontal).
+    ntiles_y : int, optional (deprecated in favor of ``kernel_size``)
+        Number of tile regions in the Y direction (vertical).
     clip_limit : float: optional
         Clipping limit, normalized between 0 and 1 (higher values give more
         contrast).
@@ -48,14 +56,15 @@ def equalize_adapthist(image, ntiles_x=8, ntiles_y=8, clip_limit=0.01,
     out : ndarray
         Equalized image.
 
+    See Also
+    --------
+    equalize_hist, rescale_intensity
+
     Notes
     -----
-    * The algorithm relies on an image whose rows and columns are even
-      multiples of the number of tiles, so the extra rows and columns are left
-      at their original values, thus  preserving the input image shape.
     * For color images, the following steps are performed:
-       - The image is converted to LAB color space
-       - The CLAHE algorithm is run on the L channel
+       - The image is converted to HSV color space
+       - The CLAHE algorithm is run on the V (Value) channel
        - The image is converted back to RGB space and returned
     * For RGBA images, the original alpha channel is removed.
 
@@ -64,38 +73,39 @@ def equalize_adapthist(image, ntiles_x=8, ntiles_y=8, clip_limit=0.01,
     .. [1] http://tog.acm.org/resources/GraphicsGems/gems.html#gemsvi
     .. [2] https://en.wikipedia.org/wiki/CLAHE#CLAHE
     """
-    args = [None, ntiles_x, ntiles_y, clip_limit * nbins, nbins]
-    if image.ndim > 2:
-        lab_img = color.rgb2lab(skimage.img_as_float(image))
-        l_chan = lab_img[:, :, 0]
-        l_chan /= np.max(np.abs(l_chan))
-        l_chan = skimage.img_as_uint(l_chan)
-        args[0] = rescale_intensity(l_chan, out_range=(0, NR_OF_GREY - 1))
-        new_l = _clahe(*args).astype(float)
-        new_l = rescale_intensity(new_l, out_range=(0, 100))
-        lab_img[:new_l.shape[0], :new_l.shape[1], 0] = new_l
-        image = color.lab2rgb(lab_img)
-        image = rescale_intensity(image, out_range=(0, 1))
-    else:
-        image = skimage.img_as_uint(image)
-        args[0] = rescale_intensity(image, out_range=(0, NR_OF_GREY - 1))
-        out = _clahe(*args)
-        image[:out.shape[0], :out.shape[1]] = out
-        image = rescale_intensity(image)
-    return image
+    image = img_as_uint(image)
+    image = rescale_intensity(image, out_range=(0, NR_OF_GREY - 1))
+
+    if ntiles_x is not None or ntiles_y is not None:
+        warn('`ntiles_*` have been deprecated in favor of '
+             '`kernel_size`.  The `ntiles_*` keyword arguments '
+             'will be removed in v0.14', skimage_deprecation)
+
+    if kernel_size is None:
+        ntiles_x = ntiles_x or 8
+        ntiles_y = ntiles_y or 8
+        kernel_size = (np.round(image.shape[0] / ntiles_y),
+                       np.round(image.shape[1] / ntiles_x))
+
+    if isinstance(kernel_size, numbers.Number):
+        kernel_size = (kernel_size, kernel_size)
+
+    kernel_size = [int(k) for k in kernel_size]
+
+    image = _clahe(image, kernel_size, clip_limit * nbins, nbins)
+    image = img_as_float(image)
+    return rescale_intensity(image)
 
 
-def _clahe(image, ntiles_x, ntiles_y, clip_limit, nbins=128):
+def _clahe(image, kernel_size, clip_limit, nbins=128):
     """Contrast Limited Adaptive Histogram Equalization.
 
     Parameters
     ----------
     image : array-like
         Input image.
-    ntiles_x : int, optional
-        Number of tile regions in the X direction.  Ranges between 2 and 16.
-    ntiles_y : int, optional
-        Number of tile regions in the Y direction.  Ranges between 2 and 16.
+    kernel_size: 2-tuple
+        Defines the shape of contextual regions used in the algorithm.
     clip_limit : float, optional
         Normalized clipping limit (higher values give more contrast).
     nbins : int, optional
@@ -112,91 +122,87 @@ def _clahe(image, ntiles_x, ntiles_y, clip_limit, nbins=128):
     minimum and maximum value as the input image. A clip limit smaller than 1
     results in standard (non-contrast limited) AHE.
     """
-    ntiles_x = min(ntiles_x, MAX_REG_X)
-    ntiles_y = min(ntiles_y, MAX_REG_Y)
-    ntiles_y = max(ntiles_x, 2)
-    ntiles_x = max(ntiles_y, 2)
 
     if clip_limit == 1.0:
         return image  # is OK, immediately returns original image.
 
-    map_array = np.zeros((ntiles_y, ntiles_x, nbins), dtype=int)
+    nr = int(np.ceil(image.shape[0] / kernel_size[0]))
+    nc = int(np.ceil(image.shape[1] / kernel_size[1]))
 
-    y_res = image.shape[0] - image.shape[0] % ntiles_y
-    x_res = image.shape[1] - image.shape[1] % ntiles_x
-    image = image[: y_res, : x_res]
+    row_step = int(np.floor(image.shape[0] / nr))
+    col_step = int(np.floor(image.shape[1] / nc))
 
-    x_size = image.shape[1] // ntiles_x  # Actual size of contextual regions
-    y_size = image.shape[0] // ntiles_y
-    n_pixels = x_size * y_size
+    bin_size = 1 + NR_OF_GREY // nbins
+    lut = np.arange(NR_OF_GREY)
+    lut //= bin_size
 
-    if clip_limit > 0.0:  # Calculate actual cliplimit
-        clip_limit = int(clip_limit * (x_size * y_size) / nbins)
-        if clip_limit < 1:
-            clip_limit = 1
-    else:
-        clip_limit = NR_OF_GREY  # Large value, do not clip (AHE)
-
-    bin_size = 1 + NR_OF_GREY / nbins
-    aLUT = np.arange(NR_OF_GREY)
-    aLUT //= bin_size
-    img_blocks = view_as_blocks(image, (y_size, x_size))
+    map_array = np.zeros((nr, nc, nbins), dtype=int)
 
     # Calculate greylevel mappings for each contextual region
-    for y in range(ntiles_y):
-        for x in range(ntiles_x):
-            sub_img = img_blocks[y, x]
-            hist = aLUT[sub_img.ravel()]
+    for r in range(nr):
+        for c in range(nc):
+            sub_img = image[r * row_step: (r + 1) * row_step,
+                            c * col_step: (c + 1) * col_step]
+
+            if clip_limit > 0.0:  # Calculate actual cliplimit
+                clim = int(clip_limit * sub_img.size / nbins)
+                if clim < 1:
+                    clim = 1
+            else:
+                clim = NR_OF_GREY  # Large value, do not clip (AHE)
+
+            hist = lut[sub_img.ravel()]
             hist = np.bincount(hist)
             hist = np.append(hist, np.zeros(nbins - hist.size, dtype=int))
-            hist = clip_histogram(hist, clip_limit)
-            hist = map_histogram(hist, 0, NR_OF_GREY - 1, n_pixels)
-            map_array[y, x] = hist
+            hist = clip_histogram(hist, clim)
+            hist = map_histogram(hist, 0, NR_OF_GREY - 1, sub_img.size)
+            map_array[r, c] = hist
 
     # Interpolate greylevel mappings to get CLAHE image
-    ystart = 0
-    for y in range(ntiles_y + 1):
-        xstart = 0
-        if y == 0:  # special case: top row
-            ystep = y_size / 2.0
-            yU = 0
-            yB = 0
-        elif y == ntiles_y:  # special case: bottom row
-            ystep = y_size / 2.0
-            yU = ntiles_y - 1
-            yB = yU
+    rstart = 0
+    for r in range(nr + 1):
+        cstart = 0
+        if r == 0:  # special case: top row
+            r_offset = row_step / 2.0
+            rU = 0
+            rB = 0
+        elif r == nr:  # special case: bottom row
+            r_offset = row_step / 2.0
+            rU = nr - 1
+            rB = rU
         else:  # default values
-            ystep = y_size
-            yU = y - 1
-            yB = yB + 1
+            r_offset = row_step
+            rU = r - 1
+            rB = rB + 1
 
-        for x in range(ntiles_x + 1):
-            if x == 0:  # special case: left column
-                xstep = x_size / 2.0
-                xL = 0
-                xR = 0
-            elif x == ntiles_x:  # special case: right column
-                xstep = x_size / 2.0
-                xL = ntiles_x - 1
-                xR = xL
+        for c in range(nc + 1):
+            if c == 0:  # special case: left column
+                c_offset = col_step / 2.0
+                cL = 0
+                cR = 0
+            elif c == nc:  # special case: right column
+                c_offset = col_step / 2.0
+                cL = nc - 1
+                cR = cL
             else:  # default values
-                xstep = x_size
-                xL = x - 1
-                xR = xL + 1
+                c_offset = col_step
+                cL = c - 1
+                cR = cL + 1
 
-            mapLU = map_array[yU, xL]
-            mapRU = map_array[yU, xR]
-            mapLB = map_array[yB, xL]
-            mapRB = map_array[yB, xR]
+            mapLU = map_array[rU, cL]
+            mapRU = map_array[rU, cR]
+            mapLB = map_array[rB, cL]
+            mapRB = map_array[rB, cR]
 
-            xslice = np.arange(xstart, xstart + xstep)
-            yslice = np.arange(ystart, ystart + ystep)
-            interpolate(image, xslice, yslice,
-                        mapLU, mapRU, mapLB, mapRB, aLUT)
+            cslice = np.arange(cstart, cstart + c_offset)
+            rslice = np.arange(rstart, rstart + r_offset)
 
-            xstart += xstep  # set pointer on next matrix */
+            interpolate(image, cslice, rslice,
+                        mapLU, mapRU, mapLB, mapRB, lut)
 
-        ystart += ystep
+            cstart += c_offset  # set pointer on next matrix */
+
+        rstart += r_offset
 
     return image
 
@@ -240,16 +246,24 @@ def clip_histogram(hist, clip_limit):
     n_excess -= mid.size * clip_limit - mid.sum()
     hist[mid_mask] = clip_limit
 
+    prev_n_excess = n_excess
+
     while n_excess > 0:  # Redistribute remaining excess
         index = 0
         while n_excess > 0 and index < hist.size:
+            under_mask = hist < 0
             step_size = int(hist[hist < clip_limit].size / n_excess)
             step_size = max(step_size, 1)
             indices = np.arange(index, hist.size, step_size)
-            under = hist[indices] < clip_limit
-            hist[under] += 1
-            n_excess -= hist[under].size
+            under_mask[indices] = True
+            under_mask = (under_mask) & (hist < clip_limit)
+            hist[under_mask] += 1
+            n_excess -= under_mask.sum()
             index += 1
+        # bail if we have not distributed any excess
+        if prev_n_excess == n_excess:
+            break
+        prev_n_excess = n_excess
 
     return hist
 
@@ -284,7 +298,7 @@ def map_histogram(hist, min_val, max_val, n_pixels):
 
 
 def interpolate(image, xslice, yslice,
-                mapLU, mapRU, mapLB, mapRB, aLUT):
+                mapLU, mapRU, mapLB, mapRB, lut):
     """Find the new grayscale level for a region using bilinear interpolation.
 
     Parameters
@@ -295,7 +309,7 @@ def interpolate(image, xslice, yslice,
        Indices of the region.
     map* : ndarray
         Mappings of greylevels from histograms.
-    aLUT : ndarray
+    lut : ndarray
         Maps grayscale levels in image to histogram levels.
 
     Returns
@@ -317,7 +331,7 @@ def interpolate(image, xslice, yslice,
 
     view = image[int(yslice[0]):int(yslice[-1] + 1),
                  int(xslice[0]):int(xslice[-1] + 1)]
-    im_slice = aLUT[view]
+    im_slice = lut[view]
     new = ((y_inv_coef * (x_inv_coef * mapLU[im_slice]
                           + x_coef * mapRU[im_slice])
             + y_coef * (x_inv_coef * mapLB[im_slice]
