@@ -2,11 +2,13 @@ from ._find_contours import find_contours
 from ._marching_cubes import (marching_cubes, mesh_surface_area,
                               correct_mesh_orientation)
 from ._regionprops import regionprops, perimeter
-from ._structural_similarity import structural_similarity
+from .simple_metrics import compare_mse, compare_nrmse, compare_psnr
+from ._structural_similarity import compare_ssim, structural_similarity
 from ._polygon import approximate_polygon, subdivide_polygon
+from ._pnpoly import points_in_poly, grid_points_in_poly
 from ._moments import moments, moments_central, moments_normalized, moments_hu
 from .profile import profile_line
-from .fit import LineModel, CircleModel, EllipseModel, ransac
+from .fit import LineModel, LineModelND, CircleModel, EllipseModel, ransac
 from .block import block_reduce
 from ._label import label
 
@@ -14,10 +16,10 @@ from ._label import label
 __all__ = ['find_contours',
            'regionprops',
            'perimeter',
-           'structural_similarity',
            'approximate_polygon',
            'subdivide_polygon',
            'LineModel',
+           'LineModelND',
            'CircleModel',
            'EllipseModel',
            'ransac',
@@ -30,4 +32,12 @@ __all__ = ['find_contours',
            'mesh_surface_area',
            'correct_mesh_orientation',
            'profile_line',
-           'label']
+           'label',
+           'points_in_poly',
+           'grid_points_in_poly',
+           'structural_similarity',
+           'compare_ssim',
+           'compare_mse',
+           'compare_nrmse',
+           'compare_psnr',
+           ]
