@@ -2,17 +2,18 @@
 
 import collections as coll
 import numpy as np
-from scipy import ndimage
-import warnings
+from scipy import ndimage as ndi
 
-from skimage.util import img_as_float, regular_grid
-from skimage.segmentation._slic import _slic_cython, _enforce_label_connectivity_cython
-from skimage.color import rgb2lab
+from .._shared.utils import warn
+from ..util import img_as_float, regular_grid
+from ..segmentation._slic import (_slic_cython,
+                                  _enforce_label_connectivity_cython)
+from ..color import rgb2lab
 
 
 def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
-         spacing=None, multichannel=True, convert2lab=True,
-         enforce_connectivity=False, min_size_factor=0.5, max_size_factor=3,
+         spacing=None, multichannel=True, convert2lab=None,
+         enforce_connectivity=True, min_size_factor=0.5, max_size_factor=3,
          slic_zero=False):
     """Segments image using k-means clustering in Color-(x,y,z) space.
 
@@ -24,10 +25,13 @@ def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
     n_segments : int, optional
         The (approximate) number of labels in the segmented output image.
     compactness : float, optional
-        Balances color-space proximity and image-space proximity. Higher
-        values give more weight to image-space. As `compactness` tends to
-        infinity, superpixel shapes become square/cubic. In SLICO mode, this
-        is the initial compactness.
+        Balances color proximity and space proximity. Higher values give
+        more weight to space proximity, making superpixel shapes more
+        square/cubic. In SLICO mode, this is the initial compactness.
+        This parameter depends strongly on image contrast and on the
+        shapes of objects in the image. We recommend exploring possible
+        values on a log scale, e.g., 0.01, 0.1, 1, 10, 100, before
+        refining around a chosen value.
     max_iter : int, optional
         Maximum number of iterations of k-means.
     sigma : float or (3,) array-like of floats, optional
@@ -46,9 +50,10 @@ def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
         channels or another spatial dimension.
     convert2lab : bool, optional
         Whether the input should be converted to Lab colorspace prior to
-        segmentation. For this purpose, the input is assumed to be RGB. Highly
-        recommended.
-    enforce_connectivity: bool, optional (default False)
+        segmentation. The input image *must* be RGB. Highly recommended.
+        This option defaults to ``True`` when ``multichannel=True`` *and*
+        ``image.shape[-1] == 3``.
+    enforce_connectivity: bool, optional
         Whether the generated segments are connected or not
     min_size_factor: float, optional
         Proportion of the minimum segment size to be removed with respect
@@ -57,7 +62,7 @@ def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
         Proportion of the maximum connected segment size. A value of 3 works
         in most of the cases.
     slic_zero: bool, optional
-        Run SLIC-zero, the zero-parameter mode of SLIC
+        Run SLIC-zero, the zero-parameter mode of SLIC. [2]_
 
     Returns
     -------
@@ -67,9 +72,8 @@ def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
     Raises
     ------
     ValueError
-        If:
-            - the image dimension is not 2 or 3 and `multichannel == False`, OR
-            - the image dimension is not 3 or 4 and `multichannel == True`
+        If ``convert2lab`` is set to ``True`` but the last array
+        dimension is not of length 3.
 
     Notes
     -----
@@ -92,12 +96,13 @@ def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
     .. [1] Radhakrishna Achanta, Appu Shaji, Kevin Smith, Aurelien Lucchi,
         Pascal Fua, and Sabine Süsstrunk, SLIC Superpixels Compared to
         State-of-the-art Superpixel Methods, TPAMI, May 2012.
+    .. [2] http://ivrg.epfl.ch/research/superpixels#SLICO
 
     Examples
     --------
     >>> from skimage.segmentation import slic
-    >>> from skimage.data import lena
-    >>> img = lena()
+    >>> from skimage.data import astronaut
+    >>> img = astronaut()
     >>> segments = slic(img, n_segments=100, compactness=10)
 
     Increasing the compactness parameter yields more square regions:
@@ -105,10 +110,6 @@ def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
     >>> segments = slic(img, n_segments=100, compactness=20)
 
     """
-    if enforce_connectivity is None:
-        warnings.warn('Deprecation: enforce_connectivity will default to'
-                      ' True in future versions.')
-        enforce_connectivity = False
 
     image = img_as_float(image)
     is_2d = False
@@ -137,19 +138,21 @@ def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
     if (sigma > 0).any():
         # add zero smoothing for multichannel dimension
         sigma = list(sigma) + [0]
-        image = ndimage.gaussian_filter(image, sigma)
+        image = ndi.gaussian_filter(image, sigma)
 
-    if convert2lab and multichannel:
-        if image.shape[3] != 3:
+    if multichannel and (convert2lab or convert2lab is None):
+        if image.shape[-1] != 3 and convert2lab:
             raise ValueError("Lab colorspace conversion requires a RGB image.")
-        image = rgb2lab(image)
+        elif image.shape[-1] == 3:
+            image = rgb2lab(image)
 
     depth, height, width = image.shape[:3]
 
     # initialize cluster centroids for desired number of segments
     grid_z, grid_y, grid_x = np.mgrid[:depth, :height, :width]
     slices = regular_grid(image.shape[:3], n_segments)
-    step_z, step_y, step_x = [int(s.step) for s in slices]
+    step_z, step_y, step_x = [int(s.step if s.step is not None else 1)
+                              for s in slices]
     segments_z = grid_z[slices]
     segments_y = grid_y[slices]
     segments_x = grid_x[slices]
