@@ -1,19 +1,13 @@
 import numpy as np
-from numpy.testing import assert_almost_equal, assert_equal
+from numpy.testing import assert_almost_equal, assert_equal, assert_raises
 
 import skimage.transform as tf
 from skimage.draw import line, circle_perimeter, ellipse_perimeter
+from skimage._shared._warnings import expected_warnings
+from skimage._shared.testing import test_parallel
 
 
-def append_desc(func, description):
-    """Append the test function ``func`` and append
-    ``description`` to its name.
-    """
-    func.description = func.__module__ + '.' + func.__name__ + description
-
-    return func
-
-
+@test_parallel()
 def test_hough_line():
     # Generate a test image
     img = np.zeros((100, 150), dtype=int)
@@ -39,12 +33,21 @@ def test_hough_line_angles():
     assert_equal(len(angles), 10)
 
 
+def test_hough_line_bad_input():
+    img = np.zeros(100)
+    img[10] = 1
+
+    # Expected error, img must be 2D
+    assert_raises(ValueError, tf.hough_line, img, np.linspace(0, 360, 10))
+
+
 def test_probabilistic_hough():
     # Generate a test image
     img = np.zeros((100, 100), dtype=int)
     for i in range(25, 75):
         img[100 - i, i] = 100
         img[i, i] = 100
+
     # decrease default theta sampling because similar orientations may confuse
     # as mentioned in article of Galambos et al
     theta = np.linspace(0, np.pi, 45)
@@ -56,9 +59,21 @@ def test_probabilistic_hough():
         line = list(line)
         line.sort(key=lambda x: x[0])
         sorted_lines.append(line)
+
     assert([(25, 75), (74, 26)] in sorted_lines)
     assert([(25, 25), (74, 74)] in sorted_lines)
 
+    # Execute with default theta
+    tf.probabilistic_hough_line(img, line_length=10, line_gap=3)
+
+
+def test_probabilistic_hough_bad_input():
+    img = np.zeros(100)
+    img[10] = 1
+
+    # Expected error, img must be 2D
+    assert_raises(ValueError, tf.probabilistic_hough_line, img)
+
 
 def test_hough_line_peaks():
     img = np.zeros((100, 150), dtype=int)
@@ -74,6 +89,21 @@ def test_hough_line_peaks():
     assert_almost_equal(theta[0], 1.41, 1)
 
 
+def test_hough_line_peaks_ordered():
+    # Regression test per PR #1421
+    testim = np.zeros((256, 64), dtype=np.bool)
+
+    testim[50:100, 20] = True
+    testim[85:200, 25] = True
+    testim[15:35, 50] = True
+    testim[1:-1, 58] = True
+
+    hough_space, angles, dists = tf.hough_line(testim)
+
+    hspace, _, _ = tf.hough_line_peaks(hough_space, angles, dists)
+    assert hspace[0] > hspace[1]
+
+
 def test_hough_line_peaks_dist():
     img = np.zeros((100, 100), dtype=np.bool_)
     img[:, 30] = True
@@ -82,10 +112,14 @@ def test_hough_line_peaks_dist():
     assert len(tf.hough_line_peaks(hspace, angles, dists,
                                    min_distance=5)[0]) == 2
     assert len(tf.hough_line_peaks(hspace, angles, dists,
-                                   min_distance=15)[0]) == 1
+                                       min_distance=15)[0]) == 1
 
 
 def test_hough_line_peaks_angle():
+    check_hough_line_peaks_angle()
+
+
+def check_hough_line_peaks_angle():
     img = np.zeros((100, 100), dtype=np.bool_)
     img[:, 0] = True
     img[0, :] = True
@@ -120,6 +154,7 @@ def test_hough_line_peaks_num():
                                    min_angle=0, num_peaks=1)[0]) == 1
 
 
+@test_parallel()
 def test_hough_circle():
     # Prepare picture
     img = np.zeros((120, 100), dtype=int)
@@ -128,8 +163,11 @@ def test_hough_circle():
     y, x = circle_perimeter(y_0, x_0, radius)
     img[x, y] = 1
 
+    out1 = tf.hough_circle(img, radius)
+    out2 = tf.hough_circle(img, [radius])
+    assert_equal(out1, out2)
     out = tf.hough_circle(img, np.array([radius], dtype=np.intp))
-
+    assert_equal(out, out1)
     x, y = np.where(out[0] == out[0].max())
     assert_equal(x[0], x_0)
     assert_equal(y[0], y_0)
