@@ -1,28 +1,14 @@
-import warnings
-
 import numpy as np
+from ..qt import QtWidgets, has_qt, FigureManagerQT, FigureCanvasQTAgg
+from ..._shared.utils import warn
+import matplotlib as mpl
+from matplotlib.figure import Figure
+from matplotlib import _pylab_helpers
+from matplotlib.colors import LinearSegmentedColormap
 
-from ..qt import qt_api
-
-try:
-    import matplotlib as mpl
-    from matplotlib.figure import Figure
-    from matplotlib import _pylab_helpers
-    from matplotlib.colors import LinearSegmentedColormap
-    if qt_api is None:
-        raise ImportError
-    else:
-        from matplotlib.backends.backend_qt4 import FigureManagerQT
-        from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg
-    if 'agg' not in mpl.get_backend().lower():
-        print("Recommended matplotlib backend is `Agg` for full "
-              "skimage.viewer functionality.")
-except ImportError:
-    FigureCanvasQTAgg = object  # hack to prevent nosetest and autodoc errors
-    LinearSegmentedColormap = object
-    print("Could not import matplotlib -- skimage.viewer not available.")
-
-from ..qt import QtGui
+if has_qt and 'agg' not in mpl.get_backend().lower():
+    warn("Recommended matplotlib backend is `Agg` for full "
+         "skimage.viewer functionality.")
 
 
 __all__ = ['init_qtapp', 'start_qtapp', 'RequiredAttr', 'figimage',
@@ -39,9 +25,9 @@ def init_qtapp():
     The QApplication needs to be initialized before creating any QWidgets
     """
     global QApp
-    QApp = QtGui.QApplication.instance()
+    QApp = QtWidgets.QApplication.instance()
     if QApp is None:
-        QApp = QtGui.QApplication([])
+        QApp = QtWidgets.QApplication([])
     return QApp
 
 
@@ -72,16 +58,13 @@ class RequiredAttr(object):
 
     instances = dict()
 
-    def __init__(self, msg='Required attribute not set', init_val=None):
+    def __init__(self, init_val=None):
         self.instances[self, None] = init_val
-        self.msg = msg
 
     def __get__(self, obj, objtype):
         value = self.instances[self, obj]
         if value is None:
-            # Should raise an error but that causes issues with the buildbot.
-            warnings.warn(self.msg)
-            self.__set__(obj, self.init_val)
+            raise AttributeError('Required attribute not set')
         return value
 
     def __set__(self, obj, value):
@@ -131,8 +114,8 @@ class FigureCanvas(FigureCanvasQTAgg):
         self.fig = figure
         FigureCanvasQTAgg.__init__(self, self.fig)
         FigureCanvasQTAgg.setSizePolicy(self,
-                                        QtGui.QSizePolicy.Expanding,
-                                        QtGui.QSizePolicy.Expanding)
+                                        QtWidgets.QSizePolicy.Expanding,
+                                        QtWidgets.QSizePolicy.Expanding)
         FigureCanvasQTAgg.updateGeometry(self)
 
     def resizeEvent(self, event):
@@ -206,6 +189,7 @@ def figimage(image, scale=1, dpi=None, **kwargs):
 
     ax.set_axis_off()
     ax.imshow(image, **kwargs)
+    ax.figure.canvas.draw()
     return fig, ax
 
 
